@@ -1,7 +1,6 @@
 package bucket
 
 import (
-	"context"
 	"time"
 
 	"github.com/pkg/errors"
@@ -28,12 +27,11 @@ var backoff = wait.Backoff{
 }
 
 func (p *Provisioner) getStorageClassWithBackoff(name string) (class *storagev1.StorageClass, err error) {
-	ctx := context.TODO()
 	logger.Infof("getting storage class %q", name)
 	classClient := p.context.Clientset.StorageV1().StorageClasses()
 	// Retry Get() with backoff.  Errors other than IsNotFound are ignored.
 	err = wait.ExponentialBackoff(backoff, func() (done bool, err error) {
-		class, err = classClient.Get(ctx, name, metav1.GetOptions{})
+		class, err = classClient.Get(p.clusterInfo.Context, name, metav1.GetOptions{})
 		if err == nil {
 			return true, nil
 		}
